@@ -197,6 +197,9 @@ class ElementsAccessor {
   DISALLOW_COPY_AND_ASSIGN(ElementsAccessor);
 };
 
+void CheckArrayAbuse(JSObject* obj, const char* op, uint32_t key,
+                     bool allow_appending = false);
+
 } }  // namespace v8::internal
 
 #endif  // V8_ELEMENTS_H_
