@@ -869,41 +869,43 @@ INSTANCE_TYPES = {
   142: "EXTERNAL_FLOAT_ARRAY_TYPE",
   144: "EXTERNAL_PIXEL_ARRAY_TYPE",
   146: "FILLER_TYPE",
-  147: "ACCESSOR_INFO_TYPE",
-  148: "ACCESSOR_PAIR_TYPE",
-  149: "ACCESS_CHECK_INFO_TYPE",
-  150: "INTERCEPTOR_INFO_TYPE",
-  151: "CALL_HANDLER_INFO_TYPE",
-  152: "FUNCTION_TEMPLATE_INFO_TYPE",
-  153: "OBJECT_TEMPLATE_INFO_TYPE",
-  154: "SIGNATURE_INFO_TYPE",
-  155: "TYPE_SWITCH_INFO_TYPE",
-  156: "ALLOCATION_SITE_INFO_TYPE",
-  157: "SCRIPT_TYPE",
-  158: "CODE_CACHE_TYPE",
-  159: "POLYMORPHIC_CODE_CACHE_TYPE",
-  160: "TYPE_FEEDBACK_INFO_TYPE",
-  161: "ALIASED_ARGUMENTS_ENTRY_TYPE",
-  164: "FIXED_ARRAY_TYPE",
+  147: "DECLARED_ACCESSOR_DESCRIPTOR_TYPE",
+  148: "DECLARED_ACCESSOR_INFO_TYPE",
+  149: "EXECUTABLE_ACCESSOR_INFO_TYPE",
+  150: "ACCESSOR_PAIR_TYPE",
+  151: "ACCESS_CHECK_INFO_TYPE",
+  152: "INTERCEPTOR_INFO_TYPE",
+  153: "CALL_HANDLER_INFO_TYPE",
+  154: "FUNCTION_TEMPLATE_INFO_TYPE",
+  155: "OBJECT_TEMPLATE_INFO_TYPE",
+  156: "SIGNATURE_INFO_TYPE",
+  157: "TYPE_SWITCH_INFO_TYPE",
+  158: "ALLOCATION_SITE_INFO_TYPE",
+  159: "SCRIPT_TYPE",
+  160: "CODE_CACHE_TYPE",
+  161: "POLYMORPHIC_CODE_CACHE_TYPE",
+  162: "TYPE_FEEDBACK_INFO_TYPE",
+  163: "ALIASED_ARGUMENTS_ENTRY_TYPE",
+  166: "FIXED_ARRAY_TYPE",
   145: "FIXED_DOUBLE_ARRAY_TYPE",
-  165: "SHARED_FUNCTION_INFO_TYPE",
-  166: "JS_MESSAGE_OBJECT_TYPE",
-  169: "JS_VALUE_TYPE",
-  170: "JS_DATE_TYPE",
-  171: "JS_OBJECT_TYPE",
-  172: "JS_CONTEXT_EXTENSION_OBJECT_TYPE",
-  173: "JS_MODULE_TYPE",
-  174: "JS_GLOBAL_OBJECT_TYPE",
-  175: "JS_BUILTINS_OBJECT_TYPE",
-  176: "JS_GLOBAL_PROXY_TYPE",
-  177: "JS_ARRAY_TYPE",
-  168: "JS_PROXY_TYPE",
-  180: "JS_WEAK_MAP_TYPE",
-  181: "JS_REGEXP_TYPE",
-  182: "JS_FUNCTION_TYPE",
-  167: "JS_FUNCTION_PROXY_TYPE",
-  162: "DEBUG_INFO_TYPE",
-  163: "BREAK_POINT_INFO_TYPE",
+  167: "SHARED_FUNCTION_INFO_TYPE",
+  168: "JS_MESSAGE_OBJECT_TYPE",
+  171: "JS_VALUE_TYPE",
+  172: "JS_DATE_TYPE",
+  173: "JS_OBJECT_TYPE",
+  174: "JS_CONTEXT_EXTENSION_OBJECT_TYPE",
+  175: "JS_MODULE_TYPE",
+  176: "JS_GLOBAL_OBJECT_TYPE",
+  177: "JS_BUILTINS_OBJECT_TYPE",
+  178: "JS_GLOBAL_PROXY_TYPE",
+  179: "JS_ARRAY_TYPE",
+  170: "JS_PROXY_TYPE",
+  182: "JS_WEAK_MAP_TYPE",
+  183: "JS_REGEXP_TYPE",
+  184: "JS_FUNCTION_TYPE",
+  169: "JS_FUNCTION_PROXY_TYPE",
+  164: "DEBUG_INFO_TYPE",
+  165: "BREAK_POINT_INFO_TYPE",
 }
 
 
@@ -933,20 +935,20 @@ KNOWN_MAPS = {
   0x080a9: (128, "MetaMap"),
   0x080d1: (130, "OddballMap"),
   0x080f9: (68, "AsciiSymbolMap"),
-  0x08121: (164, "FixedArrayMap"),
+  0x08121: (166, "FixedArrayMap"),
   0x08149: (132, "HeapNumberMap"),
   0x08171: (135, "FreeSpaceMap"),
   0x08199: (146, "OnePointerFillerMap"),
   0x081c1: (146, "TwoPointerFillerMap"),
   0x081e9: (131, "GlobalPropertyCellMap"),
-  0x08211: (165, "SharedFunctionInfoMap"),
+  0x08211: (167, "SharedFunctionInfoMap"),
   0x08239: (4, "AsciiStringMap"),
-  0x08261: (164, "NativeContextMap"),
+  0x08261: (166, "NativeContextMap"),
   0x08289: (129, "CodeMap"),
-  0x082b1: (164, "ScopeInfoMap"),
-  0x082d9: (164, "FixedCOWArrayMap"),
+  0x082b1: (166, "ScopeInfoMap"),
+  0x082d9: (166, "FixedCOWArrayMap"),
   0x08301: (145, "FixedDoubleArrayMap"),
-  0x08329: (164, "HashTableMap"),
+  0x08329: (166, "HashTableMap"),
   0x08351: (0, "StringMap"),
   0x08379: (64, "SymbolMap"),
   0x083a1: (1, "ConsStringMap"),
@@ -978,35 +980,37 @@ KNOWN_MAPS = {
   0x087b1: (141, "ExternalUnsignedIntArrayMap"),
   0x087d9: (142, "ExternalFloatArrayMap"),
   0x08801: (143, "ExternalDoubleArrayMap"),
-  0x08829: (164, "NonStrictArgumentsElementsMap"),
-  0x08851: (164, "FunctionContextMap"),
-  0x08879: (164, "CatchContextMap"),
-  0x088a1: (164, "WithContextMap"),
-  0x088c9: (164, "BlockContextMap"),
-  0x088f1: (164, "ModuleContextMap"),
-  0x08919: (164, "GlobalContextMap"),
-  0x08941: (166, "JSMessageObjectMap"),
+  0x08829: (166, "NonStrictArgumentsElementsMap"),
+  0x08851: (166, "FunctionContextMap"),
+  0x08879: (166, "CatchContextMap"),
+  0x088a1: (166, "WithContextMap"),
+  0x088c9: (166, "BlockContextMap"),
+  0x088f1: (166, "ModuleContextMap"),
+  0x08919: (166, "GlobalContextMap"),
+  0x08941: (168, "JSMessageObjectMap"),
   0x08969: (133, "ForeignMap"),
-  0x08991: (171, "NeanderMap"),
-  0x089b9: (156, "AllocationSiteInfoMap"),
-  0x089e1: (159, "PolymorphicCodeCacheMap"),
-  0x08a09: (157, "ScriptMap"),
-  0x08a31: (171, ""),
-  0x08a59: (171, "ExternalMap"),
-  0x08a81: (147, "AccessorInfoMap"),
-  0x08aa9: (148, "AccessorPairMap"),
-  0x08ad1: (149, "AccessCheckInfoMap"),
-  0x08af9: (150, "InterceptorInfoMap"),
-  0x08b21: (151, "CallHandlerInfoMap"),
-  0x08b49: (152, "FunctionTemplateInfoMap"),
-  0x08b71: (153, "ObjectTemplateInfoMap"),
-  0x08b99: (154, "SignatureInfoMap"),
-  0x08bc1: (155, "TypeSwitchInfoMap"),
-  0x08be9: (158, "CodeCacheMap"),
-  0x08c11: (160, "TypeFeedbackInfoMap"),
-  0x08c39: (161, "AliasedArgumentsEntryMap"),
-  0x08c61: (162, "DebugInfoMap"),
-  0x08c89: (163, "BreakPointInfoMap"),
+  0x08991: (173, "NeanderMap"),
+  0x089b9: (158, "AllocationSiteInfoMap"),
+  0x089e1: (161, "PolymorphicCodeCacheMap"),
+  0x08a09: (159, "ScriptMap"),
+  0x08a31: (173, ""),
+  0x08a59: (173, "ExternalMap"),
+  0x08a81: (147, "DeclaredAccessorDescriptorMap"),
+  0x08aa9: (148, "DeclaredAccessorInfoMap"),
+  0x08ad1: (149, "ExecutableAccessorInfoMap"),
+  0x08af9: (150, "AccessorPairMap"),
+  0x08b21: (151, "AccessCheckInfoMap"),
+  0x08b49: (152, "InterceptorInfoMap"),
+  0x08b71: (153, "CallHandlerInfoMap"),
+  0x08b99: (154, "FunctionTemplateInfoMap"),
+  0x08bc1: (155, "ObjectTemplateInfoMap"),
+  0x08be9: (156, "SignatureInfoMap"),
+  0x08c11: (157, "TypeSwitchInfoMap"),
+  0x08c39: (160, "CodeCacheMap"),
+  0x08c61: (162, "TypeFeedbackInfoMap"),
+  0x08c89: (163, "AliasedArgumentsEntryMap"),
+  0x08cb1: (164, "DebugInfoMap"),
+  0x08cd9: (165, "BreakPointInfoMap"),
 }
 
 
@@ -1017,7 +1021,7 @@ KNOWN_MAPS = {
 #
 # #define ROOT_LIST_CASE(type, name, camel_name) \
 #   if (o == heap_.name()) n = #camel_name;
-# OldSpaces spit;
+# OldSpaces spit(heap());
 # printf("KNOWN_OBJECTS = {\n");
 # for (PagedSpace* s = spit.next(); s != NULL; s = spit.next()) {
 #   HeapObjectIterator it(s);
@@ -1043,25 +1047,27 @@ KNOWN_OBJECTS = {
   ("OLD_POINTER_SPACE", 0x080f1): "NumberStringCache",
   ("OLD_POINTER_SPACE", 0x088f9): "SingleCharacterStringCache",
   ("OLD_POINTER_SPACE", 0x08b01): "StringSplitCache",
-  ("OLD_POINTER_SPACE", 0x08f09): "TerminationException",
-  ("OLD_POINTER_SPACE", 0x08f19): "MessageListeners",
-  ("OLD_POINTER_SPACE", 0x08f35): "CodeStubs",
-  ("OLD_POINTER_SPACE", 0x09b61): "NonMonomorphicCache",
-  ("OLD_POINTER_SPACE", 0x0a175): "PolymorphicCodeCache",
-  ("OLD_POINTER_SPACE", 0x0a17d): "NativesSourceCache",
-  ("OLD_POINTER_SPACE", 0x0a1bd): "EmptyScript",
-  ("OLD_POINTER_SPACE", 0x0a1f9): "IntrinsicFunctionNames",
-  ("OLD_POINTER_SPACE", 0x24a49): "SymbolTable",
-  ("OLD_DATA_SPACE", 0x08081): "EmptyFixedArray",
-  ("OLD_DATA_SPACE", 0x080a1): "NanValue",
-  ("OLD_DATA_SPACE", 0x0811d): "EmptyByteArray",
-  ("OLD_DATA_SPACE", 0x08125): "EmptyString",
-  ("OLD_DATA_SPACE", 0x08131): "EmptyDescriptorArray",
+  ("OLD_POINTER_SPACE", 0x08f09): "RegExpMultipleCache",
+  ("OLD_POINTER_SPACE", 0x09311): "TerminationException",
+  ("OLD_POINTER_SPACE", 0x09321): "MessageListeners",
+  ("OLD_POINTER_SPACE", 0x0933d): "CodeStubs",
+  ("OLD_POINTER_SPACE", 0x09fa5): "NonMonomorphicCache",
+  ("OLD_POINTER_SPACE", 0x0a5b9): "PolymorphicCodeCache",
+  ("OLD_POINTER_SPACE", 0x0a5c1): "NativesSourceCache",
+  ("OLD_POINTER_SPACE", 0x0a601): "EmptyScript",
+  ("OLD_POINTER_SPACE", 0x0a63d): "IntrinsicFunctionNames",
+  ("OLD_POINTER_SPACE", 0x0d659): "ObservationState",
+  ("OLD_POINTER_SPACE", 0x27415): "SymbolTable",
+  ("OLD_DATA_SPACE", 0x08099): "EmptyDescriptorArray",
+  ("OLD_DATA_SPACE", 0x080a1): "EmptyFixedArray",
+  ("OLD_DATA_SPACE", 0x080a9): "NanValue",
+  ("OLD_DATA_SPACE", 0x08125): "EmptyByteArray",
+  ("OLD_DATA_SPACE", 0x0812d): "EmptyString",
   ("OLD_DATA_SPACE", 0x08259): "InfinityValue",
   ("OLD_DATA_SPACE", 0x08265): "MinusZeroValue",
   ("OLD_DATA_SPACE", 0x08271): "PrototypeAccessors",
-  ("CODE_SPACE", 0x12b81): "JsEntryCode",
-  ("CODE_SPACE", 0x12c61): "JsConstructEntryCode",
+  ("CODE_SPACE", 0x0aea1): "JsEntryCode",
+  ("CODE_SPACE", 0x0b5c1): "JsConstructEntryCode",
 }
 
 
